using System;
using System.Collections;
using System.ComponentModel;
using System.Web;
using System.Web.SessionState;
using System.Xml;
using System.IO;

namespace GoogleControl 
{
	/// <summary>
	/// The GoogleControl's Global is a neutral location for the static initialize method
	/// and for the static pages Hashtable. You can move them in another class if needed. 
	/// The pages Hashtable contains RefererItem objects.
	/// The RefererItem class is also defined here.
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public class Global : System.Web.HttpApplication
	{
		/// <summary>
		/// Key: referer URL
		/// Value: RefererItem object
		/// </summary>
		public static Hashtable pages = new Hashtable();

		protected void Application_Start(Object sender, EventArgs e)
		{

		}
 
		protected void Session_Start(Object sender, EventArgs e)
		{

		}

		protected void Application_BeginRequest(Object sender, EventArgs e)
		{
		}

		/// <summary>
		/// initialize adds a RefererItem entry to pages.
		/// </summary>
		/// <param name="referer">The URL of the directory that uses GoogleControl</param>
		/// <param name="path">The path of Global.asax</param>
		/// <param name="checkRemote">if true tries to find GoogleControl remotely</param>
		public static void initialize(string referer, string path, bool checkRemote) 
		{
			lock(typeof(Global)) 
			{
				if (pages.Contains(referer))
					return;
				XmlTextReader r = null;
				RefererItem ri = new RefererItem();
				if (checkRemote) 
				{
					string propUrl = "http://" + referer + "GoogleControl.xml";
					r = new XmlTextReader(propUrl);
					r.WhitespaceHandling = WhitespaceHandling.None;
					// The cheapest way to check if the URL is valid is to try reading it.
					// If we get an exception the page has no CSS
					try 
					{
						r.ReadStartElement("google-control");
					}
					catch(Exception e) 
					{
						r = null;
					}
				}
				if (r == null) 
				{
					int pos = path.LastIndexOf("\\");
					path = path.Substring(0, pos + 1);
					string xmlPath = path + "GoogleControl.xml";
					if (!File.Exists(xmlPath)) 
					{
						// Add a fake RefererItem
						ri.license = "offline";
						pages.Add(referer, ri);
						return;
					}
					r = new XmlTextReader(xmlPath);
					r.WhitespaceHandling = WhitespaceHandling.None;
					r.ReadStartElement("google-control");
				}
				ri.license = r.ReadElementString("license");
				ri.css = r.ReadElementString("css");
				ri.titleCss = r.ReadElementString("title-css");
				ri.tableMinWidth = r.ReadElementString("table-min-width");
				ri.tableMaxWidth = r.ReadElementString("table-max-width");
				ri.tableCss = r.ReadElementString("table-css");
				ri.dgHeaderCss = r.ReadElementString("datagrid-header-css");
				ri.dgCss = r.ReadElementString("datagrid-css");
				ri.dgAltCss = r.ReadElementString("datagrid-alt-css");
				ri.statusCss = r.ReadElementString("status-css");
				ri.copyrightCss = r.ReadElementString("copyright-css");
				r.ReadEndElement();
				r.Close();
				pages.Add(referer, ri);
			}
		}

		protected void Application_EndRequest(Object sender, EventArgs e)
		{

		}

		protected void Session_End(Object sender, EventArgs e)
		{

		}

		protected void Application_End(Object sender, EventArgs e)
		{

		}
	}
	/// <summary>
	/// Value in the pages Hashtable.
	/// Same information as GoogleControl.xml.
	/// </summary>
	public sealed class RefererItem 
	{
		/// <summary>
		/// CSS URL
		/// </summary>
		public string css = null;
		/// <summary>
		/// CSS class to use for the title
		/// </summary>
		public string titleCss = null;
		/// <summary>
		/// CSS class to use for the status
		/// </summary>
		public string statusCss = null;
		/// <summary>
		/// CSS class to use for the copyright
		/// </summary>
		public string copyrightCss = null;
		/// <summary>
		/// Table min width (when the Datagrids are not displayed)
		/// </summary>
		public string tableMinWidth = null;
		/// <summary>
		/// Table max width (when the Datagrids are displayed)
		/// </summary>
		public string tableMaxWidth = null;
		/// <summary>
		/// CSS class to use for the Datagrid header
		/// </summary>
		public string dgHeaderCss = null;
		/// <summary>
		/// CSS class to use for the Datagrid items
		/// </summary>
		public string dgCss = null;
		/// <summary>
		/// CSS class to use for the Datagrid alternate items
		/// </summary>
		public string dgAltCss = null;
		/// <summary>
		/// CSS class to use for the embedding table
		/// </summary>
		public string tableCss = null;
		public string license = null;
	}
}

