using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace GoogleControl
{
	/// <summary>
	/// Example of HTTP control.
	/// Calls the Google Web API.
	/// <p>Copyright (c) 2002 Alexis Grandemange<br/>
	/// Mail: alexis.grandemange@pagebox.net</p>
	/// <pre>This program is free software; you can redistribute it and/or
	/// modify it under the terms of the GNU Lesser General Public
	/// License as published by the Free Software Foundation; version
	/// 2.1 of the License.
	/// This library is distributed in the hope that it will be useful,
	/// but WITHOUT ANY WARRANTY; without even the implied warranty of
	/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	/// GNU Lesser General Public License for more details.
	/// A copy of the GNU Lesser General Public License lesser.txt should be
	/// included in the distribution.</pre>
	/// </summary>
	public class WebForm1 : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Label title;
		protected System.Web.UI.WebControls.TextBox term;
		protected System.Web.UI.WebControls.Button search;
		protected System.Web.UI.WebControls.Label copyright;
		protected System.Web.UI.WebControls.DataGrid result;
		protected System.Web.UI.WebControls.DataGrid detail;
		protected System.Web.UI.WebControls.Button clear;
	
		/// <summary>
		/// Dynamically created status label (used in OnSearch).
		/// </summary>
		Label labStatus = null;
		/// <summary>
		/// Array bound with the result Datagrid.
		/// Contains ResultItem objects. 
		/// </summary>
		ArrayList results;
		/// <summary>
		/// Set by bind() and used by OnSearch().
		/// Used to display Web service invocation errors.
		/// </summary>
		string bindError = null;

		public WebForm1()
		{
			Page.Init += new System.EventHandler(Page_Init);
		}

		/// <summary>
		/// Call initialize, set the title and copyright CSS classes
		/// and resize the table to its min value.
		/// </summary>
		private void Page_Load(object sender, System.EventArgs e)
		{
			if (!IsPostBack) 
			{
				string path = Request.ServerVariables["PATH_TRANSLATED"];
				string referer = getReferer();
				Global.initialize(referer, path, true);
				RefererItem ri = null;
				ri = (RefererItem)Global.pages[referer];
				if (ri.titleCss != null)
					title.CssClass = ri.titleCss;
				if (ri.copyrightCss != null)
					copyright.CssClass = ri.copyrightCss;
				resize(getForm(), referer, false);
			}
		}

		/// <summary>
		/// If the control was included retrieve the including page from the Session object.
		/// If not found in the session object (first time) use the query string if it is set
		/// and store in the session.
		/// If the control is invoked standalone (neither session or query string set) use this URL.
		/// </summary>
		/// <returns>The URL of the directory that uses the control</returns>
		private string getReferer() 
		{
			string referer = (string)Session["referer"];
			if (referer != null)
				return referer;
			referer = Request.QueryString["from"];
			if (referer == null) 
			{
				referer = Request.ServerVariables["HTTP_HOST"] + Request.ServerVariables["URL"];
				int pos = referer.LastIndexOf('/');
				if (pos == -1)
					return null;
				referer = referer.Substring(0, pos + 1);
			}
			else 
			{
				int pos = referer.LastIndexOf('/');
				if (pos == -1)
					return null;
				referer = referer.Substring(0, pos + 1);
				Session.Add("referer", referer);
			}
			return referer;
		}

		/// <summary>
		/// Finds the GoogleForm form in the page.
		/// </summary>
		/// <returns>The GoogleForm form</returns>
		private HtmlForm getForm() 
		{
			IEnumerator cie = Controls.GetEnumerator();
			while(cie.MoveNext()) 
			{
				Object o = cie.Current;
				if (!(o is HtmlForm))
					continue;
				HtmlForm hf = (HtmlForm)o;
				if (hf.ID.Equals("Form1")) 
					return hf;
			}
			return null;
		}

		/// <summary>
		/// 1. Retrieves the consumer GoogleControl.xml info
		/// 2. Find the table in the form
		/// 3. Sets its size and CSS class
		/// </summary>
		/// <param name="hf">GoogleForm form</param>
		/// <param name="referer">URL of the consumer directory (key in Global.pages)</param>
		/// <param name="toMax">If true sets the table width to its max value.
		/// If false sets the table width to its min value</param>
		private void resize(HtmlForm hf, string referer, bool toMax) 
		{
			RefererItem ri = (RefererItem)Global.pages[referer];
			if ((ri.tableMinWidth == null) || (ri.tableMaxWidth == null) ||
				(ri.tableCss == null))
				return;
			IEnumerator ie = hf.Controls.GetEnumerator();
			while(ie.MoveNext()) 
			{
				Object o = ie.Current;
				if (o is LiteralControl) 
				{
					LiteralControl lc = (LiteralControl)o;
					string s = lc.Text;
					if (s.IndexOf("<table>", 0) >= 0) 
					{
						if (toMax)
							lc.Text = s.Replace("<table>", "<table width=\"" + ri.tableMaxWidth +
								"\" class=\"" + ri.tableCss + "\">");
						else
							lc.Text = s.Replace("<table>", "<table width=\"" + ri.tableMinWidth +
								"\" class=\"" + ri.tableCss + "\">");
					}
				}
			}
		}

		private void Page_Init(object sender, EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
		}

		#region Web Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.search.Click += new System.EventHandler(this.OnSearch);
			this.clear.Click += new System.EventHandler(this.OnClear);
			this.result.ItemCommand += new System.Web.UI.WebControls.DataGridCommandEventHandler(this.dgResult_Select);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		/// <summary>
		/// Called when the user clicks on the Search button.
		/// Calls bind.
		/// In case of success, binds the result Datagrid and set its header,
		/// item and alternate item CSS classes.
		/// </summary>
		private void OnSearch(object sender, System.EventArgs e)
		{
			HtmlForm hf = getForm();
			if (hf == null)
				return;
			hf.Target = "_self";
			ControlCollection controls = hf.Controls;
			if (controls == null)
				return;
			labStatus = new Label();
			string referer = getReferer();
			RefererItem ri = (RefererItem)Global.pages[referer];
			labStatus.ID = "labStatus";
			if (ri.statusCss != null)
				labStatus.CssClass = ri.statusCss;
			controls.Add(labStatus);
			if (term.Text.Length == 0)
				labStatus.Text = "You must set the search string";
			else 
			{
				unbindDetail();
				int resultNb = bind();
				if (resultNb == -1) 
					labStatus.Text = bindError;
				else 
				{
					resize(hf, referer, true);
					result.DataSource = results;
					result.DataBind();
					labStatus.Text = "Found " + resultNb + " entries";
					if (ri.dgHeaderCss != null)
						result.HeaderStyle.CssClass = ri.dgHeaderCss;
					if (ri.dgCss != null)
						result.ItemStyle.CssClass = ri.dgCss;
					if (ri.dgAltCss != null)
						result.AlternatingItemStyle.CssClass = ri.dgAltCss;
				}
			}
		}

		/// <summary>
		/// Called when the user clicks on a Sel button in the result Datagrid.
		/// Binds the detail Datagrid and set its header,
		/// item and alternate item CSS classes.
		/// </summary>
		private void dgResult_Select(object source, System.Web.UI.WebControls.DataGridCommandEventArgs e) 
		{
			int sel = e.Item.ItemIndex;
			if (sel != -1) 
			{
				string Key = (string)result.DataKeys[sel];
				string[] fields = Key.Split(new char[]{ '|' });
				ArrayList al = new ArrayList();
				DetailItem di = new DetailItem("Summary", fields[0]);
				al.Add(di);
				di = new DetailItem("URL", fields[1]);
				al.Add(di);
				di = new DetailItem("Snippet", fields[2]);
				al.Add(di);
				di = new DetailItem("Title", fields[3]);
				al.Add(di);
				di = new DetailItem("Dir title", fields[5]);
				al.Add(di);
				di = new DetailItem("Cache size", fields[4]);
				al.Add(di);
				detail.DataSource = al;
				detail.DataBind();
				string referer = getReferer();
				RefererItem ri = (RefererItem)Global.pages[referer];
				if (ri.dgHeaderCss != null) 
				{
					result.HeaderStyle.CssClass = ri.dgHeaderCss;
					detail.HeaderStyle.CssClass = ri.dgHeaderCss;
				}
				if (ri.dgCss != null) 
				{
					result.ItemStyle.CssClass = ri.dgCss;
					detail.ItemStyle.CssClass = ri.dgCss;
				}
				if (ri.dgAltCss != null) 
				{
					result.AlternatingItemStyle.CssClass = ri.dgAltCss;
					detail.AlternatingItemStyle.CssClass = ri.dgAltCss;
				}
				resize(getForm(), referer, true);
			}
		}

		/// <summary>
		/// Calls the Google Web service and populates the result array list.
		/// Supports a special license, offline.
		/// If you don't the license in GoogleControl.xml or if you don't define
		/// GoogleControl.xml, the Web service is not called and results is populated
		/// with dummy entries. Useful for offline tests.
		/// </summary>
		/// <returns>Number of entries found</returns>
		private int bind() 
		{
			string referer = getReferer();
			RefererItem ri = (RefererItem)Global.pages[referer];
			if (ri.license.Equals("offline")) 
			{
				results = new ArrayList(10);
				for (int i = 0; i < 10; ++ i) 
				{
					string idx = Convert.ToString(i);
					ResultItem resi = new ResultItem("summary" + idx, "http://localhost/home2/home.html",
						"snippet" + idx, "title" + idx, idx, "Directory Title" + idx);
					results.Add(resi);
				}
				return 10;
			}
			GoogleSearchService gss = new GoogleSearchService();
			try 
			{
				// gss.Discover();
				GoogleSearchResult gsr = gss.doGoogleSearch(ri.license, term.Text, 
					0, 10, true, "", true, "", "", "");
				int resultNb = gsr.resultElements.Length;
				results = new ArrayList(resultNb);
				IEnumerator ie = gsr.resultElements.GetEnumerator();
				while(ie.MoveNext()) 
				{
					ResultElement re = (ResultElement)ie.Current;
					ResultItem resi = new ResultItem(re.summary, re.URL, re.snippet, re.title,
						re.cachedSize, re.directoryTitle);
					results.Add(resi);
				}
				return resultNb;
			}
			catch(Exception e) 
			{
				bindError = e.Message;
			}
			return -1;
		}

		/// <summary>
		/// Called when the user clicks on the Clear button.
		/// Shrinks the control (clear the Datagrids).
		/// </summary>
		private void OnClear(object sender, System.EventArgs e)
		{
			unbindDetail();
			result.DataSource = null;
			result.DataBind();
			string referer = getReferer();
			resize(getForm(), referer, false);
		}

		/// <summary>
		/// Clear the detail Datagrid.
		/// </summary>
		private void unbindDetail() 
		{
			detail.DataSource = null;
			detail.DataBind();
		}
	}

	/// <summary>
	/// Describes a row of the result Datagrid.
	/// </summary>
	public sealed class ResultItem 
	{
		/// <summary>
		/// Summary info returned by the Google Web service.
		/// </summary>
		public string summary;
		/// <summary>
		/// URL returned by the Google Web service.
		/// </summary>
		public string url;
		/// <summary>
		/// Snippet returned by the Google Web service.
		/// </summary>
		public string snippet;
		/// <summary>
		/// Title returned by the Google Web service.
		/// </summary>
		public string title;
		/// <summary>
		/// Cached size returned by the Google Web service.
		/// </summary>
		public string cachedSize;
		/// <summary>
		/// Title of the page in a directory returned by the Google Web service.
		/// </summary>
		public string directoryTitle;
		/// <summary>
		/// Constructor. Set the member fields.
		/// </summary>
		/// <param name="summary">Summary info returned by the Google Web service</param>
		/// <param name="URL">URL returned by the Google Web service</param>
		/// <param name="snippet">Snippet returned by the Google Web service</param>
		/// <param name="title">Title returned by the Google Web service</param>
		/// <param name="cachedSize">Cached size returned by the Google Web service</param>
		/// <param name="directoryTitle">Title of the page in a directory
		/// returned by the Google Web service</param>
		public ResultItem(string summary, string URL, string snippet, string title, string cachedSize,
			string directoryTitle) 
		{
			this.summary = summary;
			this.url = URL;
			this.snippet = snippet;
			this.title = title;
			this.cachedSize = cachedSize;
			this.directoryTitle = directoryTitle;
		}

		/// <summary>
		/// URL property. Displayed with a hyperlink.
		/// </summary>
		public string URL 
		{
			get { return url; }
		}

		/// <summary>
		/// Title property. Displayed on a column.
		/// </summary>
		public string Title
		{
			get { return title; }
		}

		/// <summary>
		/// Key property. Contains all fields. Used to display details.
		/// </summary>
		public string Key
		{
			get 
			{
				return summary + '|' + url + '|' + snippet + '|' + title + '|' + cachedSize +
					"|" + directoryTitle;
			}
		}
	}

	/// <summary>
	/// Describes a row in the detail Datagrid.
	/// </summary>
	public sealed class DetailItem 
	{
		/// <summary>
		/// Name of a field - for instance Summary.
		/// </summary>
		private string name;
		/// <summary>
		/// Value of a field.
		/// </summary>
		private string val;
		/// <summary>
		/// Constructor. Set the member variables.
		/// </summary>
		/// <param name="name">Name of a field</param>
		/// <param name="val">Value of a field</param>
		public DetailItem(string name, string val) 
		{
			this.name = name;
			this.val = val;
		}

		/// <summary>
		/// Returns the name of a field.
		/// </summary>
		public string Name 
		{
			get { return name; }
		}

		/// <summary>
		/// Returns the value of a field,
		/// </summary>
		public string Value
		{
			get { return val; }
		}
	}
}
